FRAMEWORK_SLUGS = lambda: ['pytorch-1.0.0', 'tensorflow-1.4.0', 'tensorflow-1.5.0', 'tensorflow-1.3.0', 'tensorflow-1.2.0', 'tensorflow-1.0.0']
INSTANCE_SLUGS = lambda: ['c4.2xlarge', 'c4.2xlarge-spot', 'p2.xlarge', 'p2.xlarge-spot', 'p3.2xlarge', 't2.small']
PS_SLUGS = lambda: ['c4.2xlarge', 'c4.2xlarge-spot', 'p2.xlarge-spot', 't2.small']

