from collections import OrderedDict

GET_DATASET_TEST = OrderedDict(
    [('id', 'ab3d71a8-6c31-40fc-a761-d1b5f7b32729'), ('display_name', 'test_kwiatek'), ('icon', 'fa-btc'),
     ('color', '#47c5d1'), ('name', 'test_kwiatek'), ('source', 'gitlab'), ('full_name', 'smeler/test_kwiatek'),
     ('dataset', 'ab3d71a8-6c31-40fc-a761-d1b5f7b32729'), ('parameters', OrderedDict()), ('description', ''),
     ('http_url_to_repo', 'https://git.tensorport.com/smeler/test_kwiatek.git'), ('git_auth_link',
                                                                                  'https://smeler:28ea32b920da06b2350531cd6d26de17ab231906@git.tensorport.com/smeler/test_kwiatek.git'),
     ('parent_repository', None), ('parent_repository_name', None), ('parent_repository_owner', None),
     ('created_by', 269), ('members', [OrderedDict([('id', 269), ('access_level', 'owner'), ('username', 'smeler'), (
    'photo_url', 'https://tensorport-api-development.s3.amazonaws.com/images/empty_image_400_400.png'),
                                                    ('email', 'smeler@gormanet.pl'), ('first_name', ''),
                                                    ('last_name', ''), ('profile_url', 'QA3Y0F86M0XW'),
                                                    ('date_joined', '2017-07-24T15:15:49Z'),
                                                    ('last_login', '2018-04-05T08:06:13.447233Z')])]), ('commits', []),
     ('files', None), ('owner', OrderedDict([('id', 269), ('access_level', 'owner'), ('username', 'smeler'), (
    'photo_url', 'https://tensorport-api-development.s3.amazonaws.com/images/empty_image_400_400.png'),
                                             ('email', 'smeler@gormanet.pl'), ('first_name', ''), ('last_name', ''),
                                             ('profile_url', 'QA3Y0F86M0XW'), ('date_joined', '2017-07-24T15:15:49Z'),
                                             ('last_login', '2018-04-05T08:06:13.447233Z')])), ('is_public', False),
     ('created_at', '2018-03-29T12:37:28.907855Z'), ('modified_at', '2018-04-11T13:47:25.121852Z'), ('tags', []),
     ('size', 0)])
